﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

// See https://en.wikipedia.org/wiki/Weierstrass_function

namespace howto_weierstrass_function
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private float A = 0.5f;
        private int B, MinB;

        private void Form1_Load(object sender, EventArgs e)
        {
            scrA.Maximum = 99 + scrA.LargeChange - 1;
            SetMinB();
            DrawGraph();
        }

        private void SetMinB()
        {
            MinB = (int)Math.Ceiling((1 + 1.5 * Math.PI) / A);
            if (MinB % 2  == 0) MinB++;
            lblB.Text = "(odd ≥ " + MinB.ToString() + ")";
            if (B < MinB)
            {
                B = MinB;
                txtB.Text = B.ToString();
            }
        }

        private void scrA_Scroll(object sender, ScrollEventArgs e)
        {
            A = scrA.Value / 100f;  
            lblA.Text = A.ToString("0.00");

            SetMinB();
            DrawGraph();
        }

        private void txtB_TextChanged(object sender, EventArgs e)
        {
            int b;
            if ((int.TryParse(txtB.Text, out b)) &&
                (b >= MinB) &&
                (b % 2 == 1))
            {
                B = b;
                DrawGraph();
            }
        }

        // Draw the graph.
        private void DrawGraph()
        {
            const float wxmin = -2;
            const float wxmax = 2;
            const float wymin = -2;
            const float wymax = 2;

            int wid = picGraph.ClientSize.Width;
            int hgt = picGraph.ClientSize.Height;
            Bitmap bm = new Bitmap(wid, hgt);
            using (Graphics gr = Graphics.FromImage(bm))
            {
                gr.SmoothingMode = SmoothingMode.AntiAlias;

                RectangleF rect = new RectangleF(
                    wxmin, wymin, (wxmax - wxmin), (wymax - wymin));
                PointF[] points =
                {
                    new PointF(0, hgt),
                    new PointF(wid, hgt),
                    new PointF(0, 0),
                };
                gr.Transform = new Matrix(rect, points);
                Matrix inverse = gr.Transform;
                inverse.Invert();

                // Draw the axes.
                using (Pen pen = new Pen(Color.Red, 0))
                {
                    gr.DrawLine(pen, wxmin, 0, wxmax, 0);
                    gr.DrawLine(pen, 0, wymin, 0, wymax);
                }

                // Plot the function.
                // Convert X coordinates for each pixel into world coordinates.
                PointF[] values = new PointF[wid];
                for (int i = 0; i < wid; i++) values[i].X = i;
                inverse.TransformPoints(values);

                // Generate Y values.
                for (int i = 0; i < wid; i++)
                    values[i].Y = F(values[i].X, A, B);

                // Plot.
                using (Pen pen = new Pen(Color.Black, 0))
                {
                    gr.DrawLines(pen, values);
                }
            }

            picGraph.Image = bm;
        }

        // Return a value of the Weierstrass function.
        private float F(float x, float A, float B)
        {
            const int iterations = 100;
            double total = 0;
            for (int n = 0; n < iterations; n++)
            {
                double cos = Math.Cos(Math.Pow(B, n) * Math.PI * x);
                if (cos > 1) cos = 0;
                else if (cos < -1) cos = 0;

                total += Math.Pow(A, n) * cos;
            }
            return (float)total;
        }
    }
}
